wap("The school cafeteria ordered 42 red apples and 7 green apples for students lunches . But , if only 9 students wanted fruit , how many extra did the cafeteria end up with ?",42,7,9,40).
wap("Rachel bought two coloring books . One had 23 pictures and the other had 32 . After one week she had already colored 44 of the pictures . How many pictures does she still have to color ?",23,32,44,11).
wap("Ned had to wash 9 short sleeve shirts and 21 long sleeve shirts before school . If he had only washed 29 of them by the time school started , how many did he not wash ?",9,21,29,1).
wap("There were 39 girls and 4 boys trying out for the schools basketball team . If only 26 of them got called back , how many students did n't make the cut ?",39,4,26,17).
wap("While on vacation , Rachel took 6 pictures at the zoo and 9 at the museum . If she later deleted 11 of the pictures , how many pictures from her vacation did she still have ?",6,9,11,4).
wap("The school cafeteria ordered 33 red apples and 23 green apples for students lunches . But , if only 21 students wanted fruit , how many extra did the cafeteria end up with ?",33,23,21,35).
wap("A pet store had 41 siamese cats and 28 house cats . During a sale they sold 15 cats . How many cats do they have left ?",41,28,15,54).
wap("Janet picked 4 tulips and 11 roses to make flower bouquets . If she only used 11 of the flowers though , how many extra flowers did Janet pick ?",4,11,11,4).
wap("Vanessa had 13 music files and 30 video files on her flash drive . If she deleted 10 of the files , how many files were still on her flash drive ?",13,30,10,33).
wap("Olivia and her mom were picking carrots from their garden . Olivia picked 20 and her mother picked 14 . If only 19 of the carrots were good , how many bad carrots did they have ?",20,14,19,15).
wap("George had 30 dollars . For his birthday he got 16 more dollars but spent 38 on a new game . How much money does he have now ?",30,16,38,8).
wap("At the schools book fair Victor bought 32 adventure books and 37 mystery books . If 16 of the books were used , how many new books did he buy ?",32,37,16,53).
wap("Nancy and her mom were picking carrots from their garden . Nancy picked 38 and her mother picked 47 . If only 71 of the carrots were good , how many bad carrots did they have ?",38,47,71,14).
wap("Tom had to wash 10 short sleeve shirts and 25 long sleeve shirts before school . If he had only washed 5 of them by the time school started , how many did he not wash ?",10,25,5,30).
wap("Vanessa had 16 music files and 48 video files on her flash drive . If she deleted 30 of the files , how many files were still on her flash drive ?",16,48,30,34).
wap("Kaleb started his own lawn mowing business . In the spring he made 4 dollars mowing lawns and in the summer he made 50 dollars . If he had to spend 4 dollars buying supplies , how much money did he end up with ?",4,50,4,50).
wap("While shopping , Maria bought 40 green towels and 44 white towels . If she gave her mother 65 of them , how many towels did Maria end up with ?",40,44,65,19).
wap("For Halloween Katie and her sister combined the candy they received . Katie had 8 pieces of candy while her sister had 23 . If they ate 8 pieces the first night , how many pieces do they have left ?",8,23,8,23).
wap("For Halloween Faye scored 47 pieces of candy . She ate 25 pieces the first night and then her sister gave her 40 more pieces . How many pieces of candy does Faye have now ?",47,25,40,62).
wap("A book store had 41 books in the bargin bin . If they sold 33 books , but then put 2 more in the bin , how many books would be in the bin ?",41,33,2,10).
wap("At the fair there were 9 people in line for the bumper cars . If 6 of them got tired of waiting and left and 3 more got in line , how many people would be in line ?",9,6,3,6).
wap("A waiter had 47 customers to wait on . If 41 customers left and he got another 20 customers , how many customers would he have ?",47,41,20,26).
wap("A book store had 4 books in the bargin bin . If they sold 3 books , but then put 10 more in the bin , how many books would be in the bin ?",4,3,10,11).
wap("For the school bake sale Bianca made 14 cupcakes . If she sold 6 of them and then made 17 more , how many cupcakes would she have ?",14,6,17,25).
wap("Zoe had 15 songs on her mp3 player . If she deleted 8 old songs from it and then added 50 new songs , how many songs does she have on her mp3 player ?",15,8,50,57).
wap("For the school bake sale Carol made 19 cupcakes . If she sold 6 of them and then made 27 more , how many cupcakes would she have ?",19,6,27,40).
wap("Maria had 14 bottles of water in her fridge . If she drank 8 of them and then bought 45 more , how many bottles would she have ?",14,8,45,51).
wap("A teacher had 34 worksheets to grade . If she graded 7 , but then another 36 were turned in , how many worksheets would she have to grade ?",34,7,36,63).
wap("At the fair there were 7 people in line for the bumper cars . If 4 of them got tired of waiting and left and 8 more got in line , how many people would be in line ?",7,4,8,11).
wap("The school cafeteria had 17 apples . If they used 2 to make lunch for the students and then bought 23 more , how many apples would they have ?",17,2,23,38).
wap("Bianca had 34 songs on her mp3 player . If she deleted 14 old songs from it and then added 44 new songs , how many songs does she have on her mp3 player ?",34,14,44,64).
wap("In fourth grade there were 8 students at the start of the year . During the year 5 students left and 8 new students came to school . How many students were in fourth grade at the end ?",8,5,8,11).
wap("In fourth grade there were 31 students at the start of the year . During the year 5 students left and 11 new students came to school . How many students were in fourth grade at the end ?",31,5,11,37).
wap("Debby had 30 coloring books . If she gave away 7 of them , but then bought 35 more , how many would she have total ?",30,7,35,58).
wap("The school cafeteria had 12 apples . If they used 8 to make lunch for the students and then bought 19 more , how many apples would they have ?",12,8,19,23).
wap("A florist had 50 roses . If she sold 15 of them and then later picked 21 more , how many roses would she have ?",50,15,21,56).
wap("A store had 40 oranges in a bin . If they threw away 25 of the old ones and put 21 new ones in the bin how many would be in the bin ?",40,25,21,36).
wap("At the arcade Edward won 9 tickets . If he spent 4 tickets on a beanie and later won 4 more tickets , how many would he have ?",9,4,4,9).
wap("At the town carnival Billy rode the ferris wheel 7 times and the bumper cars 3 times . If each ride cost 5 tickets , how many tickets did he use ?",7,3,5,50).
wap("Chloe was unboxing some of her old winter clothes . She found 4 boxes of clothing and inside each box there were 2 scarves and 6 mittens . How many pieces of winter clothing did Chloe have total ?",4,2,6,32).
wap("Haley 's favorite band was holding a concert where tickets were 4 dollars each . Haley bought 3 tickets for herself and her friends and 5 extra tickets in case anyone else wanted to go . How much did she spend ?",4,3,5,32).
wap("April 's discount flowers was having a sale where each flower was 3 dollars . If Zoe bought 8 roses and 2 daisies , how much did she spend ?",3,8,2,30).
wap("Sam was collecting cans for recycling . On Saturday he filled 4 bags up and on Sunday he filled 3 more bags . If each bag had 6 cans in it , how many cans did he pick up total ?",4,3,6,42).
wap("At the town carnival Oliver rode the ferris wheel 5 times and the bumper cars 4 times . If each ride cost 7 tickets , how many tickets did he use ?",5,4,7,63).
wap("Chloe was organizing her book case making sure each of the shelves had exactly 6 books on it . If she had 5 shelves of mystery books and 4 shelves of picture books , how many books did she have total ?",6,5,4,54).
wap("Zoe was unboxing some of her old winter clothes . She found 8 boxes of clothing and inside each box there were 4 scarves and 6 mittens . How many pieces of winter clothing did Zoe have total ?",8,4,6,80).
wap("There were 2 friends playing a video game online when 2 more players joined the game . If each player had 3 lives , how many lives did they have total ?",2,2,3,12).
wap("While shopping for music online , Zoe bought 3 country albums and 5 pop albums . Each album came with a lyric sheet and had 3 songs . How many songs did Zoe buy total ?",3,5,3,24).
wap("Megan was organizing her book case making sure each of the shelves had exactly 7 books on it . If she had 8 shelves of mystery books and 2 shelves of picture books , how many books did she have total ?",7,8,2,70).
wap("Isabel was selling her necklaces at a garage sale . She sold 3 bead necklaces and 3 gem stone necklaces . If each necklace cost 6 dollars , how much money did she earn ?",3,3,6,36).
wap("April 's discount flowers was having a sale where each flower was 8 dollars . If Vanessa bought 3 roses and 3 daisies , how much did she spend ?",8,3,3,48).
wap("Kaleb was collecting cans for recycling . On Saturday he filled 5 bags up and on Sunday he filled 5 more bags . If each bag had 4 cans in it , how many cans did he pick up total ?",5,5,4,40).
wap("Chloe was playing a video game where she scores 9 points for each treasure she finds . If she found 6 treasures on the first level and 3 on the second , what would her score be ?",9,6,3,81).
wap("Lana was unboxing some of her old winter clothes . She found 3 boxes of clothing and inside each box there were 3 scarves and 4 mittens . How many pieces of winter clothing did Lana have total ?",3,3,4,21).
wap("There were 8 friends playing a video game online when 2 more players joined the game . If each player had 6 lives , how many lives did they have total ?",8,2,6,60).
wap("Nancy was organizing her book case making sure each of the shelves had exactly 6 books on it . If she had 2 shelves of mystery books and 6 shelves of picture books , how many books did she have total ?",6,2,6,48).
wap("While shopping for music online , Janet bought 6 country albums and 2 pop albums . Each album came with a lyric sheet and had 9 songs . How many songs did Janet buy total ?",6,2,9,72).
wap("Roger was helping the cafeteria workers pick up lunch trays , but he could only carry 4 trays at a time . If he had to pick up 10 trays from one table and 2 trays from another , how many trips will he make ?",4,10,2,3).
wap("Debby 's class is going on a field trip to the zoo . If each van can hold 4 people and there are 2 students and 6 adults going , how many vans will they need ?",4,2,6,2).
wap("Isabel uploaded 2 pictures from her phone and 4 from her camera to facebook . If she sorted the pics into 3 different albums with the same amount of pics in each album , how many pictures were in each of the albums ?",2,4,3,2).
wap("Gwen and her friends were recycling paper for their class . For every 3 pounds they recycled they earned one point . If Gwen recycled 5 pounds and her friends recycled 13 pounds , how many points did they earn ?",3,5,13,6).
wap("Nancy 's class is going on a field trip to the zoo . If each van can hold 5 people and there are 12 students and 3 adults going , how many vans will they need ?",5,12,3,3).
wap("The schools debate team had 28 boys and 4 girls on it . If they were split into groups of 4 how many groups could they make ?",28,4,4,8).
wap("A vase can hold 5 flowers . If you had 6 carnations and 19 roses , how many vases would you need to hold the flowers ?",5,6,19,5).
wap("Haley and her friends were recycling paper for their class . For every 3 pounds they recycled they earned one point . If Haley recycled 11 pounds and her friends recycled 16 pounds , how many points did they earn ?",3,11,16,9).
wap("There school 's baseball team had 12 new players and 44 returning players . If the coach put them into groups with 8 players in each group , how many groups would there be ?",12,44,8,7).
wap("Frank had saved up 3 dollars . If he received another 37 dollars for his allowance , how many 8 dollar toys could he buy ?",3,37,8,5).
wap("Robin 's class is going on a field trip to the zoo . If each van can hold 8 people and there are 22 students and 2 adults going , how many vans will they need ?",8,22,2,3).
wap("A group of 3 friends went into a restaurant . The chef already had 8 chicken wings cooked but cooked 10 more for the group . If they each got the same amount how many would each person get ?",3,8,10,6).
wap("While playing at the arcade , Ned won 26 tickets playing ` whack a mole ' and 19 tickets playing ` skee ball ' . If he was trying to buy candy that cost 9 tickets a piece , how many could he buy ?",26,19,9,5).
wap("A pet shelter had 2 puppies when another 34 were brought in . If 4 puppies a day are adopted , how long would it take for all of them to be adopted ?",2,34,4,9).
wap("Debby 's class is going on a field trip to the zoo . If each van can hold 9 people and there are 40 students and 14 adults going , how many vans will they need ?",9,40,14,6).
wap("For Halloween Emily received 5 pieces of candy from neighbors and 13 pieces from her older sister . If she only ate 9 pieces a day , how long would the candy last her ?",5,13,9,2).
wap("Jerry had 7 action figures , but needed 16 total for a complete collection . If each one costs $ 8 , how much money would he need to finish his collection ?",7,16,8,72).
wap("Mike had 16 video games but 8 of them were n't working . If he wanted to sell the working games for $ 7 each , how much money could he earn ?",16,8,7,56).
wap("A trivia team had 15 members total , but during a game 6 members did n't show up . If each member that did show up scored 3 points , how many points were scored total ?",15,6,3,27).
wap("Will bought 7 boxes of chocolate candy and gave 3 to his little brother . If each box has 4 pieces inside it , how many pieces did Will still have ?",7,3,4,16).
wap("A magician was selling magic card decks for 2 dollars each . If he started with 5 decks and by the end of the day he had 3 left , how much money did he earn ?",2,5,3,4).
wap("A painter needed to paint 9 rooms in a building . Each room takes 8 hours to paint . If he already painted 5 rooms , how much longer will he take to paint the rest ?",9,8,5,32).
wap("In a video game , each enemy defeated gives you 8 points . If a level has 7 enemies total and you destroy all but 2 of them , how many points would you earn ?",8,7,2,40).
wap("Zoe baked 5 brownies , but needed 11 total for her party . If she used 7 cups of flour on each one , how much cups of flour does she still need ?",5,11,7,42).
wap("Gwen earned 8 points for each bag of cans she recycled . If she had 4 bags , but did n't recycle 2 of them , how many points would she have earned ?",8,4,2,16).
wap("A chef needs to cook 13 potatoes . He has already cooked 5 . If each potato takes 6 minutes to cook , how long will it take him to cook the rest ?",13,5,6,48).
wap("April 's discount flowers was having a sale where each rose was 9 dollars . If April started with 11 roses and had 8 roses left , how much money did she earn ?",9,11,8,27).
wap("There were 13 friends playing a video game online when 8 players quit . If each player left had 6 lives , how many lives did they have total ?",13,8,6,30).
wap("Mike had 15 video games but 9 of them were n't working . If he wanted to sell the working games for $ 5 each , how much money could he earn ?",15,9,5,30).
wap("At a restaurant each adult meal costs $ 2 and kids eat free . If a group of 9 people came in and 2 were kids , how much would it cost for the group to eat ?",2,9,2,14).
wap("Edward bought 79 tickets at the state fair . He spent 23 tickets at the ` dunk a clown ' booth and decided to use the rest on rides . If each ride cost 7 tickets , how many rides could he go on ?",79,23,7,8).
wap("Isabel had 72 homework problems . She finished 32 of them but still had 5 pages of problems to do . If each page has the same number of problems on it , how many problems are on each page ?",72,32,5,8).
wap("A company invited 18 people to a luncheon , but 12 of them did n't show up . If the tables they had held 3 people each , how many tables do they need ?",18,12,3,2).
wap("Emily 's team won their dodgeball game and scored 39 points total . If Emily scored 23 of the points and everyone else scored 2 points each , how many players were on her team ?",39,23,2,8).
wap("A store had 40 coloring books in stock . They ended up putting them on sale and getting rid of 20 of them . The put the ones they still had onto shelves with 4 on each shelf . How many shelves did they use ?",40,20,4,5).
wap("A store had 120 coloring books in stock . They ended up putting them on sale and getting rid of 39 of them . The put the ones they still had onto shelves with 9 on each shelf . How many shelves did they use ?",120,39,9,9).
wap("A company invited 45 people to a luncheon , but 35 of them did n't show up . If the tables they had held 2 people each , how many tables do they need ?",45,35,2,5).
wap("Wendy picked 45 flowers for her friend 's wedding . She was making bouquets with 5 flowers in each one . If 35 of the flowers wilted before the wedding , how many bouquets could she still make ?",45,5,35,2).
wap("Sarah had 55 homework problems . She finished 6 of them but still had 7 pages of problems to do . If each page has the same number of problems on it , how many problems are on each page ?",55,6,7,7).
wap("A waiter had 22 customers in his section . If 14 of them left and the rest of his tables had 4 people at each table , how many tables did he have ?",22,14,4,2).
wap("A store had 27 coloring books in stock . They ended up putting them on sale and getting rid of 6 of them . The put the ones they still had onto shelves with 7 on each shelf . How many shelves did they use ?",27,6,7,3).
wap("Haley was planting vegetables in her garden . She started with 56 seeds and planted 35 of them in the big garden and in each of her small gardens put 3 seeds each . How many small gardens did Haley have ?",56,35,3,7).
wap("Katie had 85 files on her computer . She deleted 40 of them and put the rest into folders with 5 files in each one . How many folders did Katie end up with ?",85,40,5,9).