wap("For Halloween Debby and her sister combined the candy they received . Debby had 32 pieces of candy while her sister had 42 . If they ate 35 pieces the first night , how many pieces do they have left ?",32,42,35,39).
wap("Luke was trying to expand his game collection . He bought 2 games from a friend and bought 2 more at a garage sale . If 2 of the games did n't work , how many good games did he end up with ?",2,2,2,2).
wap("Lana picked 36 tulips and 37 roses to make flower bouquets . If she only used 70 of the flowers though , how many extra flowers did Lana pick ?",36,37,70,3).
wap("Carol and her mom were picking carrots from their garden . Carol picked 29 and her mother picked 16 . If only 38 of the carrots were good , how many bad carrots did they have ?",29,16,38,7).
wap("Roger had 16 dollars . For his birthday he got 28 more dollars but spent 25 on a new game . How much money does he have now ?",16,28,25,19).
wap("Oliver had to wash 39 short sleeve shirts and 47 long sleeve shirts before school . If he had only washed 20 of them by the time school started , how many did he not wash ?",39,47,20,66).
wap("For the school bake sale Wendy made pastries . She baked 4 cupcakes and 29 cookies . After the sale she had 24 to take back home . How many pastries did she sell ?",4,29,24,9).
wap("While on vacation , Debby took 24 pictures at the zoo and 12 at the museum . If she later deleted 14 of the pictures , how many pictures from her vacation did she still have ?",24,12,14,22).
wap("Faye had 46 math problems and 9 science problems for homework . If she finished 40 of the problems at school , how many problems did she have to do for homework ?",46,9,40,15).
wap("Amy had 4 music files and 21 video files on her flash drive . If she deleted 23 of the files , how many files were still on her flash drive ?",4,21,23,2).
wap("Ned was trying to expand his game collection . He bought 11 games from a friend and bought 22 more at a garage sale . If 19 of the games did n't work , how many good games did he end up with ?",11,22,19,14).
wap("Chloe was playing a trivia game . In the first round she scored 40 points and in the second round she scored 50 points . In the last round she lost 4 points . How many points did she have at the end of the game ?",40,50,4,86).
wap("At the arcade , Tom won 32 tickets playing ` whack a mole ' and 25 tickets playing ` skee ball ' . If he spent 7 of his tickets on a hat , how many tickets does Tom have left ?",32,25,7,50).
wap("Zoe bought two coloring books . One had 44 pictures and the other had 44 . After one week she had already colored 20 of the pictures . How many pictures does she still have to color ?",44,44,20,68).
wap("John was trying to expand his game collection . He bought 21 games from a friend and bought 8 more at a garage sale . If 23 of the games did n't work , how many good games did he end up with ?",21,8,23,6).
wap("While on vacation , Gwen took 41 pictures at the zoo and 29 at the museum . If she later deleted 15 of the pictures , how many pictures from her vacation did she still have ?",41,29,15,55).
wap("Sam had to wash 40 short sleeve shirts and 23 long sleeve shirts before school . If he had only washed 29 of them by the time school started , how many did he not wash ?",40,23,29,34).
wap("A pet store had 12 siamese cats and 20 house cats . During a sale they sold 20 cats . How many cats do they have left ?",12,20,20,12).
wap("Faye and her mom were picking carrots from their garden . Faye picked 23 and her mother picked 5 . If only 12 of the carrots were good , how many bad carrots did they have ?",23,5,12,16).
wap("The school cafeteria ordered 43 red apples and 32 green apples for students lunches . But , if only 2 students wanted fruit , how many extra did the cafeteria end up with ?",43,32,2,73).
wap("A pet store had 38 siamese cats and 25 house cats . During a sale they sold 45 cats . How many cats do they have left ?",38,25,45,18).
wap("Carol was playing a trivia game . In the first round she scored 17 points and in the second round she scored 6 points . In the last round she lost 16 points . How many points did she have at the end of the game ?",17,6,16,7).
wap("For Halloween Janet and her sister combined the candy they received . Janet had 34 pieces of candy while her sister had 33 . If they ate 4 pieces the first night , how many pieces do they have left ?",34,33,4,63).
wap("Haley and her mom were picking carrots from their garden . Haley picked 39 and her mother picked 38 . If only 64 of the carrots were good , how many bad carrots did they have ?",39,38,64,13).
wap("There were 30 girls and 36 boys trying out for the schools basketball team . If only 10 of them got called back , how many students did n't make the cut ?",30,36,10,56).
wap("Roger had 29 dollars . For his birthday he got 20 more dollars but spent 34 on a new game . How much money does he have now ?",29,20,34,15).
wap("At the schools book fair Sam bought 13 adventure books and 17 mystery books . If 15 of the books were used , how many new books did he buy ?",13,17,15,15).
wap("Oliver had 35 dollars . For his birthday he got 50 more dollars but spent 84 on a new game . How much money does he have now ?",35,50,84,1).
wap("Emily was playing a trivia game . In the first round she scored 16 points and in the second round she scored 33 points . In the last round she lost 48 points . How many points did she have at the end of the game ?",16,33,48,1).
wap("The school cafeteria ordered 33 red apples and 23 green apples for students lunches . But , if only 21 students wanted fruit , how many extra did the cafeteria end up with ?",33,23,21,35).
wap("Dave had to wash 9 short sleeve shirts and 27 long sleeve shirts before school . If he had only washed 20 of them by the time school started , how many did he not wash ?",9,27,20,16).
wap("Vanessa and her mom were picking carrots from their garden . Vanessa picked 17 and her mother picked 14 . If only 24 of the carrots were good , how many bad carrots did they have ?",17,14,24,7).
wap("At the schools book fair Sam bought 13 adventure books and 17 mystery books . If 15 of the books were used , how many new books did he buy ?",13,17,15,15).
wap("Amy had 26 music files and 36 video files on her flash drive . If she deleted 48 of the files , how many files were still on her flash drive ?",26,36,48,14).
wap("Oliver had 35 dollars . For his birthday he got 50 more dollars but spent 84 on a new game . How much money does he have now ?",35,50,84,1).
wap("Emily was playing a trivia game . In the first round she scored 16 points and in the second round she scored 33 points . In the last round she lost 48 points . How many points did she have at the end of the game ?",16,33,48,1).
wap("Dave had to wash 9 short sleeve shirts and 27 long sleeve shirts before school . If he had only washed 20 of them by the time school started , how many did he not wash ?",9,27,20,16).
wap("Vanessa and her mom were picking carrots from their garden . Vanessa picked 17 and her mother picked 14 . If only 24 of the carrots were good , how many bad carrots did they have ?",17,14,24,7).
wap("Debby bought two coloring books . One had 16 pictures and the other had 40 . After one week she had already colored 33 of the pictures . How many pictures does she still have to color ?",16,40,33,23).
wap("Edward started his own lawn mowing business . In the spring he made 2 dollars mowing lawns and in the summer he made 27 dollars . If he had to spend 5 dollars buying supplies , how much money did he end up with ?",2,27,5,24).
wap("There were 6 girls and 48 boys trying out for the schools basketball team . If only 7 of them got called back , how many students did n't make the cut ?",6,48,7,47).
wap("For the school bake sale Amy made pastries . She baked 15 cupcakes and 48 cookies . After the sale she had 12 to take back home . How many pastries did she sell ?",15,48,12,51).
wap("While on vacation , Haley took 50 pictures at the zoo and 8 at the museum . If she later deleted 38 of the pictures , how many pictures from her vacation did she still have ?",50,8,38,20).
wap("The school cafeteria ordered 6 red apples and 15 green apples for students lunches . But , if only 5 students wanted fruit , how many extra did the cafeteria end up with ?",6,15,5,16).
wap("While on vacation , Nancy took 49 pictures at the zoo and 8 at the museum . If she later deleted 38 of the pictures , how many pictures from her vacation did she still have ?",49,8,38,19).
wap("A pet store had 15 siamese cats and 49 house cats . During a sale they sold 19 cats . How many cats do they have left ?",15,49,19,45).
wap("There were 9 girls and 14 boys trying out for the schools basketball team . If only 2 of them got called back , how many students did n't make the cut ?",9,14,2,21).
wap("Haley had 27 music files and 42 video files on her flash drive . If she deleted 11 of the files , how many files were still on her flash drive ?",27,42,11,58).
wap("A waiter at ` The Greasy Spoon ' restaurant had 39 customers to wait on . During the lunch rush he added another 12 customers . If 49 of the customers did n't leave him a tip , how many customers did leave a tip ?",39,12,49,2).
wap("A pet store had 19 siamese cats and 45 house cats . During a sale they sold 56 cats . How many cats do they have left ?",19,45,56,8).
wap("A waiter at ` The Greasy Spoon ' restaurant had 26 customers to wait on . During the lunch rush he added another 27 customers . If 27 of the customers did n't leave him a tip , how many customers did leave a tip ?",26,27,27,26).
wap("While shopping , Emily bought 5 green towels and 30 white towels . If she gave her mother 26 of them , how many towels did Emily end up with ?",5,30,26,9).
wap("Ned was trying to expand his game collection . He bought 50 games from a friend and bought 27 more at a garage sale . If 74 of the games did n't work , how many good games did he end up with ?",50,27,74,3).
wap("For the school bake sale Wendy made pastries . She baked 41 cupcakes and 31 cookies . After the sale she had 32 to take back home . How many pastries did she sell ?",41,31,32,40).
wap("The school cafeteria ordered 25 red apples and 17 green apples for students lunches . But , if only 10 students wanted fruit , how many extra did the cafeteria end up with ?",25,17,10,32).
wap("Cody had 45 dollars . For his birthday he got 9 more dollars but spent 19 on a new game . How much money does he have now ?",45,9,19,35).
wap("A teacher had 6 worksheets to grade . If she graded 4 , but then another 18 were turned in , how many worksheets would she have to grade ?",6,4,18,20).
wap("A teacher had 7 worksheets to grade . If she graded 2 , but then another 46 were turned in , how many worksheets would she have to grade ?",7,2,46,51).
wap("Paige had 8 songs on her mp3 player . If she deleted 5 old songs from it and then added 30 new songs , how many songs does she have on her mp3 player ?",8,5,30,33).
wap("In fourth grade there were 33 students at the start of the year . During the year 18 students left and 14 new students came to school . How many students were in fourth grade at the end ?",33,18,14,29).
wap("For the school bake sale Katie made 26 cupcakes . If she sold 20 of them and then made 20 more , how many cupcakes would she have ?",26,20,20,26).
wap("Adam had 5 dollars . At the store he spent $ 2 on a new game . If he got another 5 dollars for his allowance , how much money does he have now ?",5,2,5,8).
wap("Tiffany was playing a video game and had 43 lives . In a hard part of the game she lost 14 lives . If she got 27 more lives in the next level , how many lives would she have ?",43,14,27,56).
wap("At the fair there were 30 people in line for the bumper cars . If 10 of them got tired of waiting and left and 5 more got in line , how many people would be in line ?",30,10,5,25).
wap("A teacher had 38 worksheets to grade . If she graded 4 , but then another 15 were turned in , how many worksheets would she have to grade ?",38,4,15,49).
wap("In fourth grade there were 40 students at the start of the year . During the year 14 students left and 26 new students came to school . How many students were in fourth grade at the end ?",40,14,26,52).
wap("Wendy was playing a video game and had 43 lives . In a hard part of the game she lost 8 lives . If she got 39 more lives in the next level , how many lives would she have ?",43,8,39,74).
wap("A store had 31 oranges in a bin . If they threw away 9 of the old ones and put 38 new ones in the bin how many would be in the bin ?",31,9,38,60).
wap("Adam had 33 books . If he sold 11 of them and used the money he earned to buy 23 new books , how many books would Adam have ?",33,11,23,45).
wap("At the fair there were 10 people in line for the bumper cars . If 2 of them got tired of waiting and left and 2 more got in line , how many people would be in line ?",10,2,2,10).
wap("For Halloween Robin scored 23 pieces of candy . She ate 7 pieces the first night and then her sister gave her 21 more pieces . How many pieces of candy does Robin have now ?",23,7,21,37).
wap("For Halloween Haley scored 33 pieces of candy . She ate 17 pieces the first night and then her sister gave her 19 more pieces . How many pieces of candy does Haley have now ?",33,17,19,35).
wap("Edward had 43 books . If he sold 19 of them and used the money he earned to buy 14 new books , how many books would Edward have ?",43,19,14,38).
wap("A teacher had 29 worksheets to grade . If she graded 25 , but then another 29 were turned in , how many worksheets would she have to grade ?",29,25,29,33).
wap("A store had 40 oranges in a bin . If they threw away 37 of the old ones and put 7 new ones in the bin how many would be in the bin ?",40,37,7,10).
wap("Sam had 10 socks . If he threw away 3 old ones that did n't fit and bought 36 new ones , how many socks would he have ?",10,3,36,43).
wap("Emily had 7 coloring books . If she gave away 2 of them , but then bought 14 more , how many would she have total ?",7,2,14,19).
wap("At the arcade Dave won 25 tickets . If he spent 22 tickets on a beanie and later won 15 more tickets , how many would he have ?",25,22,15,18).
wap("The school cafeteria had 23 apples . If they used 20 to make lunch for the students and then bought 6 more , how many apples would they have ?",23,20,6,9).
wap("Janet was playing a video game and had 38 lives . In a hard part of the game she lost 16 lives . If she got 32 more lives in the next level , how many lives would she have ?",38,16,32,54).
wap("Roger had 45 dollars in January . By March he had spent 20 dollars . If he got another 46 dollars from his mom , how much money would he have ?",45,20,46,71).
wap("Janet was playing a video game and had 47 lives . In a hard part of the game she lost 23 lives . If she got 46 more lives in the next level , how many lives would she have ?",47,23,46,70).
wap("Nancy picked 12 carrots from her garden . If she threw out 2 of them and then picked 21 more the next day , how many carrots would she have total ?",12,2,21,31).
wap("Tom had 5 books . If he sold 4 of them and used the money he earned to buy 38 new books , how many books would Tom have ?",5,4,38,39).
wap("For the school bake sale Maria made 19 cupcakes . If she sold 5 of them and then made 10 more , how many cupcakes would she have ?",19,5,10,24).
wap("A store had 50 oranges in a bin . If they threw away 40 of the old ones and put 24 new ones in the bin how many would be in the bin ?",50,40,24,34).
wap("Katie had 11 songs on her mp3 player . If she deleted 7 old songs from it and then added 24 new songs , how many songs does she have on her mp3 player ?",11,7,24,28).
wap("Haley was playing a video game and had 14 lives . In a hard part of the game she lost 4 lives . If she got 36 more lives in the next level , how many lives would she have ?",14,4,36,46).
wap("Luke had 48 dollars in January . By March he had spent 11 dollars . If he got another 21 dollars from his mom , how much money would he have ?",48,11,21,58).
wap("At the arcade Dave won 11 tickets . If he spent 5 tickets on a beanie and later won 10 more tickets , how many would he have ?",11,5,10,16).
wap("Haley picked 28 carrots from her garden . If she threw out 11 of them and then picked 9 more the next day , how many carrots would she have total ?",28,11,9,26).
wap("A florist had 5 roses . If she sold 3 of them and then later picked 34 more , how many roses would she have ?",5,3,34,36).
wap("Wendy was playing a video game and had 10 lives . In a hard part of the game she lost 6 lives . If she got 37 more lives in the next level , how many lives would she have ?",10,6,37,41).
wap("John had 5 dollars . At the store he spent $ 2 on a new game . If he got another 26 dollars for his allowance , how much money does he have now ?",5,2,26,29).
wap("For the school bake sale Chloe made 28 cupcakes . If she sold 25 of them and then made 8 more , how many cupcakes would she have ?",28,25,8,11).
wap("April 's discount flowers was having a sale where each flower was 3 dollars . If Emily bought 2 roses and 2 daisies , how much did she spend ?",3,2,2,12).
wap("Isabel had 2 pages of math homework and 4 pages of reading homework . If each page had 5 problems on it , how many problems did she have to complete total ?",2,4,5,30).
wap("There were 7 friends playing a video game online when 2 more players joined the game . If each player had 7 lives , how many lives did they have total ?",7,2,7,63).
wap("Rachel was organizing her book case making sure each of the shelves had exactly 9 books on it . If she had 6 shelves of mystery books and 2 shelves of picture books , how many books did she have total ?",9,6,2,72).
wap("A pet store has 9 bird cages . If each cage has 2 parrots and 2 parakeets in it , how many birds does the pet store have total ?",9,2,2,36).
wap("Paul was collecting cans for recycling . On Saturday he filled 6 bags up and on Sunday he filled 3 more bags . If each bag had 8 cans in it , how many cans did he pick up total ?",6,3,8,72).
wap("Gwen was organizing her book case making sure each of the shelves had exactly 9 books on it . If she had 3 shelves of mystery books and 5 shelves of picture books , how many books did she have total ?",9,3,5,72).
wap("Wendy bought 4 new chairs and 4 new tables for her house . If she spent 6 minutes on each piece furniture putting it together , how many minutes did it take her to finish ?",4,4,6,48).
wap("Paige was unboxing some of her old winter clothes . She found 6 boxes of clothing and inside each box there were 5 scarves and 5 mittens . How many pieces of winter clothing did Paige have total ?",6,5,5,60).
wap("Edward and his friend were buying trick decks from the magic shop for 9 dollars each . How much did they spend if Edward bought 4 decks and his friend bought 4 decks ?",9,4,4,72).
wap("Tom and his friend were buying trick decks from the magic shop for 8 dollars each . How much did they spend if Tom bought 3 decks and his friend bought 5 decks ?",8,3,5,64).
wap("While shopping for music online , Megan bought 2 country albums and 8 pop albums . Each album came with a lyric sheet and had 7 songs . How many songs did Megan buy total ?",2,8,7,70).
wap("Victor was working as a sacker at a grocery store where he made 6 dollars an hour . On Monday he worked 5 hours and on Tuesday he worked 5 hours . How much money did Victor make in those two days ?",6,5,5,60).
wap("Olivia was playing a video game where she scores 8 points for each treasure she finds . If she found 2 treasures on the first level and 2 on the second , what would her score be ?",8,2,2,32).
wap("A waiter had 6 tables he was waiting on , with 3 women and 5 men at each table . How many customers total did the waiter have ?",6,3,5,48).
wap("Faye was selling her necklaces at a garage sale . She sold 3 bead necklaces and 7 gem stone necklaces . If each necklace cost 7 dollars , how much money did she earn ?",3,7,7,70).
wap("There were 5 friends playing a video game online when 2 more players joined the game . If each player had 8 lives , how many lives did they have total ?",5,2,8,56).
wap("Edward was working as a sacker at a grocery store where he made 6 dollars an hour . On Monday he worked 3 hours and on Tuesday he worked 5 hours . How much money did Edward make in those two days ?",6,3,5,48).
wap("Amy was playing a video game where she scores 4 points for each treasure she finds . If she found 6 treasures on the first level and 2 on the second , what would her score be ?",4,6,2,32).
wap("While shopping for music online , Isabel bought 6 country albums and 2 pop albums . Each album came with a lyric sheet and had 9 songs . How many songs did Isabel buy total ?",6,2,9,72).
wap("At Oliver 's Restaurant a group with 2 adults and 4 children came in to eat . If each meal cost 3 dollars , how much was the bill ?",2,4,3,18).
wap("Tiffany had 6 pages of math homework and 4 pages of reading homework . If each page had 3 problems on it , how many problems did she have to complete total ?",6,4,3,30).
wap("Gwen was organizing her book case making sure each of the shelves had exactly 4 books on it . If she had 5 shelves of mystery books and 3 shelves of picture books , how many books did she have total ?",4,5,3,32).
wap("While shopping for music online , Isabel bought 4 country albums and 5 pop albums . Each album came with a lyric sheet and had 8 songs . How many songs did Isabel buy total ?",4,5,8,72).
wap("Rachel was playing a video game where she scores 9 points for each treasure she finds . If she found 5 treasures on the first level and 2 on the second , what would her score be ?",9,5,2,63).
wap("April 's discount flowers was having a sale where each flower was 6 dollars . If Katie bought 5 roses and 5 daisies , how much did she spend ?",6,5,5,60).
wap("A pet store has 9 bird cages . If each cage has 2 parrots and 6 parakeets in it , how many birds does the pet store have total ?",9,2,6,72).
wap("Tiffany was playing a video game where she scores 6 points for each treasure she finds . If she found 3 treasures on the first level and 5 on the second , what would her score be ?",6,3,5,48).
wap("Frank was collecting cans for recycling . On Saturday he filled 5 bags up and on Sunday he filled 3 more bags . If each bag had 5 cans in it , how many cans did he pick up total ?",5,3,5,40).
wap("A pet store has 4 bird cages . If each cage has 8 parrots and 2 parakeets in it , how many birds does the pet store have total ?",4,8,2,40).
wap("Bianca was organizing her book case making sure each of the shelves had exactly 8 books on it . If she had 5 shelves of mystery books and 4 shelves of picture books , how many books did she have total ?",8,5,4,72).
wap("While shopping for music online , Nancy bought 3 country albums and 5 pop albums . Each album came with a lyric sheet and had 8 songs . How many songs did Nancy buy total ?",3,5,8,64).
wap("Sam was collecting cans for recycling . On Saturday he filled 3 bags up and on Sunday he filled 4 more bags . If each bag had 9 cans in it , how many cans did he pick up total ?",3,4,9,63).
wap("April 's discount flowers was having a sale where each flower was 6 dollars . If Maria bought 7 roses and 3 daisies , how much did she spend ?",6,7,3,60).
wap("Dave was working as a sacker at a grocery store where he made 6 dollars an hour . On Monday he worked 6 hours and on Tuesday he worked 2 hours . How much money did Dave make in those two days ?",6,6,2,48).
wap("While shopping for music online , Faye bought 2 country albums and 3 pop albums . Each album came with a lyric sheet and had 6 songs . How many songs did Faye buy total ?",2,3,6,30).
wap("Tom was putting his spare change into piles . He had 2 piles of quarters and 4 piles of dimes . If each pile had 5 coins in it , how many coins did he have total ?",2,4,5,30).
wap("Lana was unboxing some of her old winter clothes . She found 4 boxes of clothing and inside each box there were 2 scarves and 6 mittens . How many pieces of winter clothing did Lana have total ?",4,2,6,32).
wap("Sarah had 4 pages of math homework and 6 pages of reading homework . If each page had 4 problems on it , how many problems did she have to complete total ?",4,6,4,40).
wap("Edward and his friend were buying trick decks from the magic shop for 6 dollars each . How much did they spend if Edward bought 3 decks and his friend bought 6 decks ?",6,3,6,54).
wap("Emily bought 4 new chairs and 2 new tables for her house . If she spent 8 minutes on each piece furniture putting it together , how many minutes did it take her to finish ?",4,2,8,48).
wap("Kaleb had saved up 21 dollars . If he received another 15 dollars for his allowance , how many 6 dollar toys could he buy ?",21,15,6,6).
wap("Olivia was making baggies of cookies with 9 cookies in each bag . If she had 13 chocolate chip cookies and 41 oatmeal cookies , how many baggies could she make ?",9,13,41,6).
wap("Luke made 9 dollars mowing lawns and 18 dollars weed eating . If he only spent 3 dollar a week , how long would the money last him ?",9,18,3,9).
wap("There school 's baseball team had 48 new players and 6 returning players . If the coach put them into groups with 6 players in each group , how many groups would there be ?",48,6,6,9).
wap("For a birthday party Jerry bought 41 regular sodas and 22 diet sodas . If his fridge would only hold 9 on each shelf , how many shelves would he fill up ?",41,22,9,7).
wap("Paige and her friends were recycling paper for their class . For every 4 pounds they recycled they earned one point . If Paige recycled 14 pounds and her friends recycled 2 pounds , how many points did they earn ?",4,14,2,4).
wap("Will was organizing his baseball cards in a binder with 3 on each page . If he had 8 new cards and 10 old cards to put in the binder , how many pages would he use ?",3,8,10,6).
wap("The schools debate team had 11 boys and 45 girls on it . If they were split into groups of 7 how many groups could they make ?",11,45,7,8).
wap("A group of 4 friends went into a restaurant . The chef already had 9 chicken wings cooked but cooked 7 more for the group . If they each got the same amount how many would each person get ?",4,9,7,4).
wap("A vase can hold 6 flowers . If you had 7 carnations and 47 roses , how many vases would you need to hold the flowers ?",6,7,47,9).
wap("A toy store had 4 giant stuffed bears in stock when they got another shipment with 10 bears in it . The put the bears onto shelves with 7 on each shelf . How many shelves did they use ?",4,10,7,2).
wap("John made 6 dollars mowing lawns and 18 dollars weed eating . If he only spent 3 dollar a week , how long would the money last him ?",6,18,3,8).
wap("Wendy 's old washing machine could only wash 8 pieces of clothing at a time . If she had to wash 39 shirts and 33 sweaters how many loads would she have to do ?",8,39,33,9).
wap("Mike made 14 dollars mowing lawns and 26 dollars weed eating . If he only spent 5 dollar a week , how long would the money last him ?",14,26,5,8).
wap("Robin uploaded 31 pictures from her phone and 5 from her camera to facebook . If she sorted the pics into 9 different albums with the same amount of pics in each album , how many pictures were in each of the albums ?",31,5,9,4).
wap("There school 's baseball team had 4 new players and 6 returning players . If the coach put them into groups with 5 players in each group , how many groups would there be ?",4,6,5,2).
wap("For a birthday party George bought 10 regular sodas and 22 diet sodas . If his fridge would only hold 4 on each shelf , how many shelves would he fill up ?",10,22,4,8).
wap("For Halloween Sarah received 66 pieces of candy from neighbors and 15 pieces from her older sister . If she only ate 9 pieces a day , how long would the candy last her ?",66,15,9,9).
wap("Henry was helping the cafeteria workers pick up lunch trays , but he could only carry 9 trays at a time . If he had to pick up 29 trays from one table and 52 trays from another , how many trips will he make ?",9,29,52,9).
wap("Tiffany uploaded 7 pictures from her phone and 13 from her camera to facebook . If she sorted the pics into 5 different albums with the same amount of pics in each album , how many pictures were in each of the albums ?",7,13,5,4).
wap("A toy store had 5 giant stuffed bears in stock when they got another shipment with 7 bears in it . The put the bears onto shelves with 6 on each shelf . How many shelves did they use ?",5,7,6,2).
wap("Megan 's class is going on a field trip to the zoo . If each van can hold 5 people and there are 25 students and 5 adults going , how many vans will they need ?",5,25,5,6).
wap("A group of 3 friends went into a restaurant . The chef already had 6 chicken wings cooked but cooked 3 more for the group . If they each got the same amount how many would each person get ?",3,6,3,3).
wap("A toy store had 6 giant stuffed bears in stock when they got another shipment with 18 bears in it . The put the bears onto shelves with 6 on each shelf . How many shelves did they use ?",6,18,6,4).
wap("Jerry was helping the cafeteria workers pick up lunch trays , but he could only carry 8 trays at a time . If he had to pick up 9 trays from one table and 7 trays from another , how many trips will he make ?",8,9,7,2).
wap("Frank had saved up 3 dollars . If he received another 37 dollars for his allowance , how many 8 dollar toys could he buy ?",3,37,8,5).
wap("Luke was organizing his baseball cards in a binder with 3 on each page . If he had 3 new cards and 9 old cards to put in the binder , how many pages would he use ?",3,3,9,4).
wap("A group of 3 friends went into a restaurant . The chef already had 6 chicken wings cooked but cooked 3 more for the group . If they each got the same amount how many would each person get ?",3,6,3,3).
wap("Olivia uploaded 5 pictures from her phone and 35 from her camera to facebook . If she sorted the pics into 8 different albums with the same amount of pics in each album , how many pictures were in each of the albums ?",5,35,8,5).
wap("Jerry was helping the cafeteria workers pick up lunch trays , but he could only carry 8 trays at a time . If he had to pick up 9 trays from one table and 7 trays from another , how many trips will he make ?",8,9,7,2).
wap("A vase can hold 5 flowers . If you had 6 carnations and 19 roses , how many vases would you need to hold the flowers ?",5,6,19,5).
wap("Haley and her friends were recycling paper for their class . For every 3 pounds they recycled they earned one point . If Haley recycled 11 pounds and her friends recycled 16 pounds , how many points did they earn ?",3,11,16,9).
wap("A pet shelter had 8 puppies when another 19 were brought in . If 3 puppies a day are adopted , how long would it take for all of them to be adopted ?",8,19,3,9).
wap("For homework Faye had 13 math problems and 2 spelling problems . If she can finish 3 problems in an hour how long will it take her to finish all the problems ?",13,2,3,5).
wap("Sam had saved up 8 dollars . If he received another 7 dollars for his allowance , how many 3 dollar toys could he buy ?",8,7,3,5).
wap("There school 's baseball team had 31 new players and 4 returning players . If the coach put them into groups with 7 players in each group , how many groups would there be ?",31,4,7,5).
wap("Robin was making baggies of cookies with 6 cookies in each bag . If she had 23 chocolate chip cookies and 25 oatmeal cookies , how many baggies could she make ?",6,23,25,8).
wap("There school 's baseball team had 2 new players and 6 returning players . If the coach put them into groups with 4 players in each group , how many groups would there be ?",2,6,4,2).
wap("Sarah 's old washing machine could only wash 5 pieces of clothing at a time . If she had to wash 43 shirts and 2 sweaters how many loads would she have to do ?",5,43,2,9).
wap("A pet shelter had 3 puppies when another 3 were brought in . If 3 puppies a day are adopted , how long would it take for all of them to be adopted ?",3,3,3,2).
wap("While playing at the arcade , Luke won 2 tickets playing ` whack a mole ' and 13 tickets playing ` skee ball ' . If he was trying to buy candy that cost 3 tickets a piece , how many could he buy ?",2,13,3,5).
wap("Paul had saved up 3 dollars . If he received another 7 dollars for his allowance , how many 5 dollar toys could he buy ?",3,7,5,2).
wap("Luke was helping the cafeteria workers pick up lunch trays , but he could only carry 4 trays at a time . If he had to pick up 20 trays from one table and 16 trays from another , how many trips will he make ?",4,20,16,9).
wap("Vanessa and her friends were recycling paper for their class . For every 9 pounds they recycled they earned one point . If Vanessa recycled 20 pounds and her friends recycled 16 pounds , how many points did they earn ?",9,20,16,4).
wap("At a company picnic 9 managers and 15 employees decided to start a game of volleyball . If they split into 8 teams how many people would be on each team ?",9,15,8,3).
wap("John was organizing his baseball cards in a binder with 3 on each page . If he had 8 new cards and 16 old cards to put in the binder , how many pages would he use ?",3,8,16,8).
wap("Wendy uploaded 22 pictures from her phone and 2 from her camera to facebook . If she sorted the pics into 4 different albums with the same amount of pics in each album , how many pictures were in each of the albums ?",22,2,4,6).
wap("A toy store had 17 giant stuffed bears in stock when they got another shipment with 10 bears in it . The put the bears onto shelves with 9 on each shelf . How many shelves did they use ?",17,10,9,3).
wap("Billy was organizing his baseball cards in a binder with 5 on each page . If he had 3 new cards and 42 old cards to put in the binder , how many pages would he use ?",5,3,42,9).
wap("For a birthday party Tom bought 4 regular sodas and 52 diet sodas . If his fridge would only hold 7 on each shelf , how many shelves would he fill up ?",4,52,7,8).
wap("Paul had saved up 4 dollars . If he received another 11 dollars for his allowance , how many 5 dollar toys could he buy ?",4,11,5,3).
wap("Katie uploaded 30 pictures from her phone and 51 from her camera to facebook . If she sorted the pics into 9 different albums with the same amount of pics in each album , how many pictures were in each of the albums ?",30,51,9,9).
wap("A new building needed 14 windows . The builder had already installed 5 of them . If it takes 4 hours to install each window , how long will it take him to install the rest ?",14,5,4,36).
wap("A chef needs to cook 16 potatoes . He has already cooked 7 . If each potato takes 5 minutes to cook , how long will it take him to cook the rest ?",16,7,5,45).
wap("Henry earned 5 dollars for each lawn he mowed . If he had 12 lawns to mow , but forgot to mow 7 of them , how much money did he actually earn ?",5,12,7,25).
wap("A trivia team had 5 members total , but during a game 2 members did n't show up . If each member that did show up scored 6 points , how many points were scored total ?",5,2,6,18).
wap("A painter needed to paint 10 rooms in a building . Each room takes 8 hours to paint . If he already painted 8 rooms , how much longer will he take to paint the rest ?",10,8,8,16).
wap("In a video game , each enemy defeated gives you 3 points . If a level has 6 enemies total and you destroy all but 2 of them , how many points would you earn ?",3,6,2,12).
wap("Wendy earned 5 points for each bag of cans she recycled . If she had 11 bags , but did n't recycle 2 of them , how many points would she have earned ?",5,11,2,45).
wap("Kaleb bought 14 boxes of chocolate candy and gave 5 to his little brother . If each box has 6 pieces inside it , how many pieces did Kaleb still have ?",14,5,6,54).
wap("In a video game , each enemy defeated gives you 9 points . If a level has 11 enemies total and you destroy all but 3 of them , how many points would you earn ?",9,11,3,72).
wap("There were 16 friends playing a video game online when 7 players quit . If each player left had 8 lives , how many lives did they have total ?",16,7,8,72).
wap("A new building needed 10 windows . The builder had already installed 6 of them . If it takes 5 hours to install each window , how long will it take him to install the rest ?",10,6,5,20).
wap("Adam earned 9 dollars for each lawn he mowed . If he had 12 lawns to mow , but forgot to mow 8 of them , how much money did he actually earn ?",9,12,8,36).
wap("A trivia team had 12 members total , but during a game 4 members did n't show up . If each member that did show up scored 8 points , how many points were scored total ?",12,4,8,64).
wap("A painter needed to paint 12 rooms in a building . Each room takes 7 hours to paint . If he already painted 5 rooms , how much longer will he take to paint the rest ?",12,7,5,49).
wap("There were 8 friends playing a video game online when 5 players quit . If each player left had 5 lives , how many lives did they have total ?",8,5,5,15).
wap("Each chocolate bar in a box cost $ 6 . If a box had 13 bars total and Zoe sold all but 6 bars , how much money would she have made ?",6,13,6,42).
wap("Mike invited 13 friends to a birthday party , but 7 could n't come . If he wanted to buy enough cupcakes so each person could have exactly 4 , how many should he buy ?",13,7,4,24).
wap("Roger earned 9 dollars for each lawn he mowed . If he had 14 lawns to mow , but forgot to mow 8 of them , how much money did he actually earn ?",9,14,8,54).
wap("A chef needs to cook 12 potatoes . He has already cooked 6 . If each potato takes 6 minutes to cook , how long will it take him to cook the rest ?",12,6,6,36).
wap("At the fair Adam bought 13 tickets . After riding the ferris wheel he had 4 tickets left . If each ticket cost 9 dollars , how much money did Adam spend riding the ferris wheel ?",13,4,9,81).
wap("Each chocolate bar in a box cost $ 4 . If a box had 8 bars total and Emily sold all but 3 bars , how much money would she have made ?",4,8,3,20).
wap("At lunch a waiter had 10 customers and 5 of them did n't leave a tip . If he got $ 3 each from the ones who did tip , how much money did he earn ?",10,5,3,15).
wap("Will invited 9 friends to a birthday party , but 4 could n't come . If he wanted to buy enough cupcakes so each person could have exactly 8 , how many should he buy ?",9,4,8,40).
wap("A magician was selling magic card decks for 5 dollars each . If he started with 14 decks and by the end of the day he had 5 left , how much money did he earn ?",5,14,5,45).
wap("Each chocolate bar in a box cost $ 3 . If a box had 9 bars total and Wendy sold all but 3 bars , how much money would she have made ?",3,9,3,18).
wap("Adam bought 13 boxes of chocolate candy and gave 7 to his little brother . If each box has 6 pieces inside it , how many pieces did Adam still have ?",13,7,6,36).
wap("Kaleb had 10 video games but 8 of them were n't working . If he wanted to sell the working games for $ 6 each , how much money could he earn ?",10,8,6,12).
wap("April 's discount flowers was having a sale where each rose was 7 dollars . If April started with 9 roses and had 4 roses left , how much money did she earn ?",7,9,4,35).
wap("At lunch a waiter had 9 customers and 5 of them did n't leave a tip . If he got $ 8 each from the ones who did tip , how much money did he earn ?",9,5,8,32).
wap("A new building needed 14 windows . The builder had already installed 8 of them . If it takes 8 hours to install each window , how long will it take him to install the rest ?",14,8,8,48).
wap("Bianca earned 5 points for each bag of cans she recycled . If she had 17 bags , but did n't recycle 8 of them , how many points would she have earned ?",5,17,8,45).
wap("Henry had 3 action figures , but needed 8 total for a complete collection . If each one costs $ 6 , how much money would he need to finish his collection ?",3,8,6,30).
wap("There were 10 friends playing a video game online when 7 players quit . If each player left had 8 lives , how many lives did they have total ?",10,7,8,24).
wap("A painter needed to paint 11 rooms in a building . Each room takes 7 hours to paint . If he already painted 2 rooms , how much longer will he take to paint the rest ?",11,7,2,63).
wap("A worksheet had 4 problems on it . If a teacher had 16 worksheets to grade and had already graded 8 of them , how many more problems does she have to grade ?",4,16,8,32).
wap("A worksheet had 2 problems on it . If a teacher had 14 worksheets to grade and had already graded 7 of them , how many more problems does she have to grade ?",2,14,7,14).
wap("April 's discount flowers was having a sale where each rose was 4 dollars . If April started with 13 roses and had 4 roses left , how much money did she earn ?",4,13,4,36).
wap("A painter needed to paint 6 rooms in a building . Each room takes 5 hours to paint . If he already painted 2 rooms , how much longer will he take to paint the rest ?",6,5,2,20).
wap("Each chocolate bar in a box cost $ 4 . If a box had 11 bars total and Vanessa sold all but 7 bars , how much money would she have made ?",4,11,7,16).
wap("A new building needed 12 windows . The builder had already installed 6 of them . If it takes 4 hours to install each window , how long will it take him to install the rest ?",12,6,4,24).
wap("Isabel baked 3 brownies , but needed 5 total for her party . If she used 5 cups of flour on each one , how much cups of flour does she still need ?",3,5,5,10).
wap("A worksheet had 3 problems on it . If a teacher had 15 worksheets to grade and had already graded 7 of them , how many more problems does she have to grade ?",3,15,7,24).
wap("Edward earned 4 dollars for each lawn he mowed . If he had 17 lawns to mow , but forgot to mow 9 of them , how much money did he actually earn ?",4,17,9,32).
wap("A new building needed 11 windows . The builder had already installed 4 of them . If it takes 8 hours to install each window , how long will it take him to install the rest ?",11,4,8,56).
wap("Tom bought 12 boxes of chocolate candy and gave 7 to his little brother . If each box has 6 pieces inside it , how many pieces did Tom still have ?",12,7,6,30).
wap("Tiffany baked 8 brownies , but needed 17 total for her party . If she used 8 cups of flour on each one , how much cups of flour does she still need ?",8,17,8,72).
wap("In a video game , each enemy defeated gives you 7 points . If a level has 11 enemies total and you destroy all but 8 of them , how many points would you earn ?",7,11,8,21).
wap("At lunch a waiter had 7 customers and 5 of them did n't leave a tip . If he got $ 3 each from the ones who did tip , how much money did he earn ?",7,5,3,6).
wap("A trivia team had 9 members total , but during a game 3 members did n't show up . If each member that did show up scored 2 points , how many points were scored total ?",9,3,2,12).
wap("There were 8 friends playing a video game online when 3 players quit . If each player left had 3 lives , how many lives did they have total ?",8,3,3,15).
wap("Will had 57 dollars . If he spent 27 bucks on a new game , how many 6 dollar toys could he buy with the money he had left ?",57,27,6,5).
wap("A pet store had 18 puppies . In one day they sold 3 of them and put the rest into cages with 5 in each cage . How many cages did they use ?",18,3,5,3).
wap("Luke had 47 pieces of clothing to wash . He put 17 of them in one load , but decided to split the rest into 5 equal loads . How many pieces of clothing could go in each of the small loads ?",47,17,5,6).
wap("Katie baked 18 cupcakes for her school 's bake sale . If her brother , Todd , ate 8 of them how many packages could she make if she put 2 cupcake in each package ?",18,8,2,5).
wap("Megan had 93 files on her computer . She deleted 21 of them and put the rest into folders with 8 files in each one . How many folders did Megan end up with ?",93,21,8,9).
wap("The cafeteria had 62 apples . For lunch they handed out 8 to students and decided to use the rest to make pies . If each pie takes 9 apples , how many pies could they make ?",62,8,9,6).
wap("Mike made 101 dollars mowing lawns over the summer . If he spent 47 dollars buying new mower blades , how many 6 dollar games could he buy with the money he had left ?",101,47,6,9).
wap("Paige 's team won their dodgeball game and scored 41 points total . If Paige scored 11 of the points and everyone else scored 6 points each , how many players were on her team ?",41,11,6,5).
wap("Oliver is at the library helping put away books . There are 46 book to put away total but a librarian takes 10 of them and leaves Oliver with the rest . If he can fit 4 books on a shelf , how many shelves will he need ?",46,10,4,9).
wap("Edward made 37 dollars mowing lawns over the summer . If he spent 21 dollars buying new mower blades , how many 2 dollar games could he buy with the money he had left ?",37,21,2,8).
wap("A pet store had 81 puppies . In one day they sold 41 of them and put the rest into cages with 8 in each cage . How many cages did they use ?",81,41,8,5).
wap("There are 65 students trying out for the school 's trivia teams . If 17 of them did n't get picked for the team and the rest were put into 8 groups , how many students would be in each group ?",65,17,8,6).
wap("Tom had 57 dollars . If he spent 49 bucks on a new game , how many 4 dollar toys could he buy with the money he had left ?",57,49,4,2).
wap("Isabel picked 66 flowers for her friend 's wedding . She was making bouquets with 8 flowers in each one . If 10 of the flowers wilted before the wedding , how many bouquets could she still make ?",66,8,10,7).
wap("The cafeteria had 96 apples . For lunch they handed out 42 to students and decided to use the rest to make pies . If each pie takes 6 apples , how many pies could they make ?",96,42,6,9).
wap("Edward was selling his old games . He started out with 35 but sold 19 of them . He packed the rest up putting 8 games into each box . How many boxes did he have to use ?",35,19,8,2).
wap("Megan had 40 homework problems . She finished 26 of them but still had 2 pages of problems to do . If each page has the same number of problems on it , how many problems are on each page ?",40,26,2,7).
wap("Wendy picked 103 flowers for her friend 's wedding . She was making bouquets with 8 flowers in each one . If 47 of the flowers wilted before the wedding , how many bouquets could she still make ?",103,8,47,7).
wap("Kaleb had 39 pieces of clothing to wash . He put 19 of them in one load , but decided to split the rest into 5 equal loads . How many pieces of clothing could go in each of the small loads ?",39,19,5,4).
wap("Mike is at the library helping put away books . There are 82 book to put away total but a librarian takes 10 of them and leaves Mike with the rest . If he can fit 9 books on a shelf , how many shelves will he need ?",82,10,9,8).
wap("Bianca uploaded 33 pictures to Facebook . She put 27 pics into one album and put the rest into 3 different albums . How many pictures were in each album ?",33,27,3,2).
wap("A company invited 47 people to a luncheon , but 7 of them did n't show up . If the tables they had held 5 people each , how many tables do they need ?",47,7,5,8).
wap("There are 64 students trying out for the school 's trivia teams . If 36 of them did n't get picked for the team and the rest were put into 4 groups , how many students would be in each group ?",64,36,4,7).
wap("Jerry is at the library helping put away books . There are 34 book to put away total but a librarian takes 7 of them and leaves Jerry with the rest . If he can fit 3 books on a shelf , how many shelves will he need ?",34,7,3,9).
wap("Will had 83 dollars . If he spent 47 bucks on a new game , how many 4 dollar toys could he buy with the money he had left ?",83,47,4,9).
wap("Bianca was planting vegetables in her garden . She started with 52 seeds and planted 40 of them in the big garden and in each of her small gardens put 2 seeds each . How many small gardens did Bianca have ?",52,40,2,6).
wap("There are 25 students trying out for the school 's trivia teams . If 15 of them did n't get picked for the team and the rest were put into 2 groups , how many students would be in each group ?",25,15,2,5).
wap("The cafeteria had 47 apples . For lunch they handed out 27 to students and decided to use the rest to make pies . If each pie takes 4 apples , how many pies could they make ?",47,27,4,5).
wap("Megan baked 68 cupcakes for her school 's bake sale . If her brother , Todd , ate 32 of them how many packages could she make if she put 6 cupcake in each package ?",68,32,6,6).
wap("Isabel uploaded 25 pictures to Facebook . She put 10 pics into one album and put the rest into 5 different albums . How many pictures were in each album ?",25,10,5,3).
wap("Sarah had 60 homework problems . She finished 20 of them but still had 5 pages of problems to do . If each page has the same number of problems on it , how many problems are on each page ?",60,20,5,8).
wap("Mike made 42 dollars mowing lawns over the summer . If he spent 10 dollars buying new mower blades , how many 8 dollar games could he buy with the money he had left ?",42,10,8,4).
wap("Will had 59 pieces of clothing to wash . He put 32 of them in one load , but decided to split the rest into 9 equal loads . How many pieces of clothing could go in each of the small loads ?",59,32,9,3).
wap("Oliver made 35 dollars mowing lawns over the summer . If he spent 7 dollars buying new mower blades , how many 4 dollar games could he buy with the money he had left ?",35,7,4,7).
wap("Emily was planting vegetables in her garden . She started with 42 seeds and planted 36 of them in the big garden and in each of her small gardens put 2 seeds each . How many small gardens did Emily have ?",42,36,2,3).
wap("Katie 's team won their dodgeball game and scored 12 points total . If Katie scored 4 of the points and everyone else scored 4 points each , how many players were on her team ?",12,4,4,2).
wap("Wendy uploaded 45 pictures to Facebook . She put 27 pics into one album and put the rest into 9 different albums . How many pictures were in each album ?",45,27,9,2).
wap("A store had 86 coloring books in stock . They ended up putting them on sale and getting rid of 37 of them . The put the ones they still had onto shelves with 7 on each shelf . How many shelves did they use ?",86,37,7,7).
wap("The cafeteria had 50 apples . For lunch they handed out 5 to students and decided to use the rest to make pies . If each pie takes 5 apples , how many pies could they make ?",50,5,5,9).
wap("Nancy had 101 homework problems . She finished 47 of them but still had 6 pages of problems to do . If each page has the same number of problems on it , how many problems are on each page ?",101,47,6,9).
wap("Paige had 27 files on her computer . She deleted 9 of them and put the rest into folders with 6 files in each one . How many folders did Paige end up with ?",27,9,6,3).
wap("For Halloween Bianca received 78 pieces of candy . She ate 30 pieces then placed the rest into piles with 8 in each pile . How many piles could she make ?",78,30,8,6).
wap("Haley uploaded 65 pictures to Facebook . She put 17 pics into one album and put the rest into 6 different albums . How many pictures were in each album ?",65,17,6,8).
wap("A pet store had 102 puppies . In one day they sold 21 of them and put the rest into cages with 9 in each cage . How many cages did they use ?",102,21,9,9).
wap("Paige picked 53 flowers for her friend 's wedding . She was making bouquets with 7 flowers in each one . If 18 of the flowers wilted before the wedding , how many bouquets could she still make ?",53,7,18,5).
wap("Luke was selling his old games . He started out with 39 but sold 19 of them . He packed the rest up putting 4 games into each box . How many boxes did he have to use ?",39,19,4,5).
wap("Haley baked 20 cupcakes for her school 's bake sale . If her brother , Todd , ate 11 of them how many packages could she make if she put 3 cupcake in each package ?",20,11,3,3).
wap("Wendy had 82 files on her computer . She deleted 37 of them and put the rest into folders with 5 files in each one . How many folders did Wendy end up with ?",82,37,5,9).
wap("Sarah was planting vegetables in her garden . She started with 21 seeds and planted 12 of them in the big garden and in each of her small gardens put 3 seeds each . How many small gardens did Sarah have ?",21,12,3,3).
wap("Nancy uploaded 51 pictures to Facebook . She put 11 pics into one album and put the rest into 8 different albums . How many pictures were in each album ?",51,11,8,5).
wap("A company invited 24 people to a luncheon , but 10 of them did n't show up . If the tables they had held 7 people each , how many tables do they need ?",24,10,7,2).
wap("Bianca 's team won their dodgeball game and scored 75 points total . If Bianca scored 45 of the points and everyone else scored 6 points each , how many players were on her team ?",75,45,6,5).
wap("For Halloween Emily received 54 pieces of candy . She ate 33 pieces then placed the rest into piles with 7 in each pile . How many piles could she make ?",54,33,7,3).
wap("A store had 48 coloring books in stock . They ended up putting them on sale and getting rid of 38 of them . The put the ones they still had onto shelves with 5 on each shelf . How many shelves did they use ?",48,38,5,2).
wap("Roger is at the library helping put away books . There are 14 book to put away total but a librarian takes 2 of them and leaves Roger with the rest . If he can fit 3 books on a shelf , how many shelves will he need ?",14,2,3,4).
wap("A pet store had 56 puppies . In one day they sold 24 of them and put the rest into cages with 4 in each cage . How many cages did they use ?",56,24,4,8).
wap("The cafeteria had 86 apples . For lunch they handed out 30 to students and decided to use the rest to make pies . If each pie takes 8 apples , how many pies could they make ?",86,30,8,7).
wap("Jerry had 60 pieces of clothing to wash . He put 40 of them in one load , but decided to split the rest into 5 equal loads . How many pieces of clothing could go in each of the small loads ?",60,40,5,4).
wap("For Halloween Bianca received 32 pieces of candy . She ate 12 pieces then placed the rest into piles with 5 in each pile . How many piles could she make ?",32,12,5,4).
wap("A pet store had 88 puppies . In one day they sold 34 of them and put the rest into cages with 6 in each cage . How many cages did they use ?",88,34,6,9).
wap("Roger had 68 dollars . If he spent 47 bucks on a new game , how many 7 dollar toys could he buy with the money he had left ?",68,47,7,3).
wap("Tom bought 40 tickets at the state fair . He spent 28 tickets at the ` dunk a clown ' booth and decided to use the rest on rides . If each ride cost 4 tickets , how many rides could he go on ?",40,28,4,3).