wap("A pet store had 13 siamese cats and 5 house cats . During a sale they sold 10 cats . How many cats do they have left ?",13,5,10,8).
wap("While on vacation , Megan took 15 pictures at the zoo and 18 at the museum . If she later deleted 31 of the pictures , how many pictures from her vacation did she still have ?",15,18,31,2).
wap("Katie picked 3 tulips and 9 roses to make flower bouquets . If she only used 10 of the flowers though , how many extra flowers did Katie pick ?",3,9,10,2).
wap("Bianca and her mom were picking carrots from their garden . Bianca picked 26 and her mother picked 15 . If only 16 of the carrots were good , how many bad carrots did they have ?",26,15,16,25).
wap("For the school bake sale Katie made pastries . She baked 7 cupcakes and 5 cookies . After the sale she had 8 to take back home . How many pastries did she sell ?",7,5,8,4).
wap("Henry had 11 dollars . For his birthday he got 18 more dollars but spent 10 on a new game . How much money does he have now ?",11,18,10,19).
wap("For the school bake sale Paige made pastries . She baked 36 cupcakes and 9 cookies . After the sale she had 4 to take back home . How many pastries did she sell ?",36,9,4,41).
wap("Gwen bought two coloring books . One had 10 pictures and the other had 39 . After one week she had already colored 13 of the pictures . How many pictures does she still have to color ?",10,39,13,36).
wap("Dave had to wash 29 short sleeve shirts and 11 long sleeve shirts before school . If he had only washed 35 of them by the time school started , how many did he not wash ?",29,11,35,5).
wap("Edward was trying to expand his game collection . He bought 41 games from a friend and bought 14 more at a garage sale . If 31 of the games did n't work , how many good games did he end up with ?",41,14,31,24).
wap("Amy had 26 music files and 36 video files on her flash drive . If she deleted 48 of the files , how many files were still on her flash drive ?",26,36,48,14).
wap("Paige had 43 math problems and 12 science problems for homework . If she finished 44 of the problems at school , how many problems did she have to do for homework ?",43,12,44,11).
wap("A pet store had 41 siamese cats and 28 house cats . During a sale they sold 15 cats . How many cats do they have left ?",41,28,15,54).
wap("Paige had 43 math problems and 12 science problems for homework . If she finished 44 of the problems at school , how many problems did she have to do for homework ?",43,12,44,11).
wap("While on vacation , Rachel took 6 pictures at the zoo and 9 at the museum . If she later deleted 11 of the pictures , how many pictures from her vacation did she still have ?",6,9,11,4).
wap("The school cafeteria ordered 8 red apples and 43 green apples for students lunches . But , if only 42 students wanted fruit , how many extra did the cafeteria end up with ?",8,43,42,9).
wap("A pet store had 36 siamese cats and 18 house cats . During a sale they sold 26 cats . How many cats do they have left ?",36,18,26,28).
wap("The school cafeteria ordered 37 red apples and 45 green apples for students lunches . But , if only 51 students wanted fruit , how many extra did the cafeteria end up with ?",37,45,51,31).
wap("While shopping , Maria bought 35 green towels and 21 white towels . If she gave her mother 34 of them , how many towels did Maria end up with ?",35,21,34,22).
wap("There were 17 girls and 32 boys trying out for the schools basketball team . If only 10 of them got called back , how many students did n't make the cut ?",17,32,10,39).
wap("A waiter at ` The Greasy Spoon ' restaurant had 29 customers to wait on . During the lunch rush he added another 20 customers . If 34 of the customers did n't leave him a tip , how many customers did leave a tip ?",29,20,34,15).
wap("Bianca picked 39 tulips and 49 roses to make flower bouquets . If she only used 81 of the flowers though , how many extra flowers did Bianca pick ?",39,49,81,7).
wap("Gwen had 18 math problems and 11 science problems for homework . If she finished 24 of the problems at school , how many problems did she have to do for homework ?",18,11,24,5).
wap("For Halloween Katie and her sister combined the candy they received . Katie had 10 pieces of candy while her sister had 6 . If they ate 9 pieces the first night , how many pieces do they have left ?",10,6,9,7).
wap("Rachel bought two coloring books . One had 24 pictures and the other had 39 . After one week she had already colored 4 of the pictures . How many pictures does she still have to color ?",24,39,4,59).
wap("At the arcade , Jerry won 29 tickets playing ` whack a mole ' and 17 tickets playing ` skee ball ' . If he spent 12 of his tickets on a hat , how many tickets does Jerry have left ?",29,17,12,34).
wap("For the school bake sale Carol made 30 cupcakes . If she sold 9 of them and then made 28 more , how many cupcakes would she have ?",30,9,28,49).
wap("Kaleb had 34 books . If he sold 17 of them and used the money he earned to buy 7 new books , how many books would Kaleb have ?",34,17,7,24).
wap("In fourth grade there were 10 students at the start of the year . During the year 4 students left and 42 new students came to school . How many students were in fourth grade at the end ?",10,4,42,48).
wap("Oliver had 33 dollars in January . By March he had spent 4 dollars . If he got another 32 dollars from his mom , how much money would he have ?",33,4,32,61).
wap("A florist had 37 roses . If she sold 16 of them and then later picked 19 more , how many roses would she have ?",37,16,19,40).
wap("A waiter had 19 customers to wait on . If 14 customers left and he got another 36 customers , how many customers would he have ?",19,14,36,41).
wap("Maria picked 48 carrots from her garden . If she threw out 11 of them and then picked 15 more the next day , how many carrots would she have total ?",48,11,15,52).
wap("A store had 34 oranges in a bin . If they threw away 20 of the old ones and put 13 new ones in the bin how many would be in the bin ?",34,20,13,27).
wap("At the fair there were 12 people in line for the bumper cars . If 10 of them got tired of waiting and left and 15 more got in line , how many people would be in line ?",12,10,15,17).
wap("A waiter had 14 customers to wait on . If 3 customers left and he got another 39 customers , how many customers would he have ?",14,3,39,50).
wap("Faye had 34 coloring books . If she gave away 3 of them , but then bought 48 more , how many would she have total ?",34,3,48,79).
wap("Paige had 11 songs on her mp3 player . If she deleted 9 old songs from it and then added 8 new songs , how many songs does she have on her mp3 player ?",11,9,8,10).
wap("At the arcade Cody won 49 tickets . If he spent 25 tickets on a beanie and later won 6 more tickets , how many would he have ?",49,25,6,30).
wap("Bianca had 45 coloring books . If she gave away 6 of them , but then bought 20 more , how many would she have total ?",45,6,20,59).
wap("At the arcade Jerry won 4 tickets . If he spent 2 tickets on a beanie and later won 47 more tickets , how many would he have ?",4,2,47,49).
wap("The school cafeteria had 38 apples . If they used 20 to make lunch for the students and then bought 28 more , how many apples would they have ?",38,20,28,46).
wap("George had 28 socks . If he threw away 4 old ones that did n't fit and bought 36 new ones , how many socks would he have ?",28,4,36,60).
wap("Rachel was playing a video game and had 10 lives . In a hard part of the game she lost 4 lives . If she got 26 more lives in the next level , how many lives would she have ?",10,4,26,32).
wap("A florist had 6 roses . If she sold 5 of them and then later picked 12 more , how many roses would she have ?",6,5,12,13).
wap("The school cafeteria had 14 apples . If they used 13 to make lunch for the students and then bought 49 more , how many apples would they have ?",14,13,49,50).
wap("In fourth grade there were 35 students at the start of the year . During the year 10 students left and 10 new students came to school . How many students were in fourth grade at the end ?",35,10,10,35).
wap("At the arcade Victor won 46 tickets . If he spent 27 tickets on a beanie and later won 39 more tickets , how many would he have ?",46,27,39,58).
wap("Chloe picked 48 carrots from her garden . If she threw out 45 of them and then picked 42 more the next day , how many carrots would she have total ?",48,45,42,45).
wap("Robin had 30 songs on her mp3 player . If she deleted 8 old songs from it and then added 10 new songs , how many songs does she have on her mp3 player ?",30,8,10,32).
wap("Megan picked 19 carrots from her garden . If she threw out 4 of them and then picked 46 more the next day , how many carrots would she have total ?",19,4,46,61).
wap("John had 33 socks . If he threw away 19 old ones that did n't fit and bought 13 new ones , how many socks would he have ?",33,19,13,27).
wap("Bianca picked 23 carrots from her garden . If she threw out 10 of them and then picked 47 more the next day , how many carrots would she have total ?",23,10,47,60).
wap("Zoe had 42 bottles of water in her fridge . If she drank 25 of them and then bought 30 more , how many bottles would she have ?",42,25,30,47).
wap("A store had 5 oranges in a bin . If they threw away 2 of the old ones and put 28 new ones in the bin how many would be in the bin ?",5,2,28,31).
wap("Adam had 48 books . If he sold 19 of them and used the money he earned to buy 38 new books , how many books would Adam have ?",48,19,38,67).
wap("For the school bake sale Robin made 42 cupcakes . If she sold 22 of them and then made 39 more , how many cupcakes would she have ?",42,22,39,59).
wap("A florist had 11 roses . If she sold 2 of them and then later picked 32 more , how many roses would she have ?",11,2,32,41).
wap("Frank had 11 dollars . At the store he spent $ 3 on a new game . If he got another 14 dollars for his allowance , how much money does he have now ?",11,3,14,22).
wap("Emily was playing a video game and had 42 lives . In a hard part of the game she lost 25 lives . If she got 24 more lives in the next level , how many lives would she have ?",42,25,24,41).
wap("At the fair there were 31 people in line for the bumper cars . If 25 of them got tired of waiting and left and 25 more got in line , how many people would be in line ?",31,25,25,31).
wap("Zoe had 6 songs on her mp3 player . If she deleted 3 old songs from it and then added 20 new songs , how many songs does she have on her mp3 player ?",6,3,20,23).
wap("A waiter had 33 customers to wait on . If 31 customers left and he got another 26 customers , how many customers would he have ?",33,31,26,28).
wap("In fourth grade there were 11 students at the start of the year . During the year 6 students left and 42 new students came to school . How many students were in fourth grade at the end ?",11,6,42,47).
wap("Oliver had 11 socks . If he threw away 4 old ones that did n't fit and bought 26 new ones , how many socks would he have ?",11,4,26,33).
wap("In fourth grade there were 4 students at the start of the year . During the year 3 students left and 42 new students came to school . How many students were in fourth grade at the end ?",4,3,42,43).
wap("Roger had 25 books . If he sold 21 of them and used the money he earned to buy 30 new books , how many books would Roger have ?",25,21,30,34).
wap("A waiter had 9 tables he was waiting on , with 7 women and 3 men at each table . How many customers total did the waiter have ?",9,7,3,90).
wap("Wendy was playing a video game where she scores 5 points for each treasure she finds . If she found 4 treasures on the first level and 3 on the second , what would her score be ?",5,4,3,35).
wap("Paul bought 6 boxes of chocolate candy and 4 boxes of caramel candy . If each box has 9 pieces inside it , how much candy did he have total ?",6,4,9,90).
wap("A pet store has 6 bird cages . If each cage has 6 parrots and 2 parakeets in it , how many birds does the pet store have total ?",6,6,2,48).
wap("Rachel was unboxing some of her old winter clothes . She found 7 boxes of clothing and inside each box there were 3 scarves and 4 mittens . How many pieces of winter clothing did Rachel have total ?",7,3,4,49).
wap("Cody bought 7 boxes of chocolate candy and 3 boxes of caramel candy . If each box has 8 pieces inside it , how much candy did he have total ?",7,3,8,80).
wap("At Billy 's Restaurant a group with 2 adults and 5 children came in to eat . If each meal cost 3 dollars , how much was the bill ?",2,5,3,21).
wap("While playing a trivia game , Adam answered 5 questions correct in the first half and 5 questions correct in the second half . If each question was worth 5 points , what was his final score ?",5,5,5,50).
wap("Luke was putting his spare change into piles . He had 5 piles of quarters and 5 piles of dimes . If each pile had 3 coins in it , how many coins did he have total ?",5,5,3,30).
wap("Victor and his friend were buying trick decks from the magic shop for 8 dollars each . How much did they spend if Victor bought 6 decks and his friend bought 2 decks ?",8,6,2,64).
wap("Katie had 7 pages of math homework and 3 pages of reading homework . If each page had 9 problems on it , how many problems did she have to complete total ?",7,3,9,90).
wap("Faye was playing a video game where she scores 7 points for each treasure she finds . If she found 2 treasures on the first level and 6 on the second , what would her score be ?",7,2,6,56).
wap("There were 2 friends playing a video game online when 2 more players joined the game . If each player had 6 lives , how many lives did they have total ?",2,2,6,24).
wap("George was working as a sacker at a grocery store where he made 5 dollars an hour . On Monday he worked 7 hours and on Tuesday he worked 2 hours . How much money did George make in those two days ?",5,7,2,45).
wap("While playing a trivia game , Frank answered 3 questions correct in the first half and 2 questions correct in the second half . If each question was worth 3 points , what was his final score ?",3,2,3,15).
wap("A pet store has 6 bird cages . If each cage has 2 parrots and 7 parakeets in it , how many birds does the pet store have total ?",6,2,7,54).
wap("Maria bought 2 new chairs and 2 new tables for her house . If she spent 8 minutes on each piece furniture putting it together , how many minutes did it take her to finish ?",2,2,8,32).
wap("Katie was selling her necklaces at a garage sale . She sold 4 bead necklaces and 3 gem stone necklaces . If each necklace cost 3 dollars , how much money did she earn ?",4,3,3,21).
wap("While playing a trivia game , Mike answered 3 questions correct in the first half and 5 questions correct in the second half . If each question was worth 3 points , what was his final score ?",3,5,3,24).
wap("A waiter had 9 tables he was waiting on , with 4 women and 3 men at each table . How many customers total did the waiter have ?",9,4,3,63).
wap("April 's discount flowers was having a sale where each flower was 3 dollars . If Robin bought 5 roses and 4 daisies , how much did she spend ?",3,5,4,27).
wap("Adam bought 2 boxes of chocolate candy and 5 boxes of caramel candy . If each box has 4 pieces inside it , how much candy did he have total ?",2,5,4,28).
wap("A pet store has 8 bird cages . If each cage has 2 parrots and 7 parakeets in it , how many birds does the pet store have total ?",8,2,7,72).
wap("Lana 's favorite band was holding a concert where tickets were 6 dollars each . Lana bought 8 tickets for herself and her friends and 2 extra tickets in case anyone else wanted to go . How much did she spend ?",6,8,2,60).
wap("At Tom 's Restaurant a group with 2 adults and 5 children came in to eat . If each meal cost 8 dollars , how much was the bill ?",2,5,8,56).
wap("Megan was selling her necklaces at a garage sale . She sold 7 bead necklaces and 3 gem stone necklaces . If each necklace cost 9 dollars , how much money did she earn ?",7,3,9,90).
wap("Roger was putting his spare change into piles . He had 3 piles of quarters and 3 piles of dimes . If each pile had 7 coins in it , how many coins did he have total ?",3,3,7,42).
wap("There were 2 friends playing a video game online when 2 more players joined the game . If each player had 8 lives , how many lives did they have total ?",2,2,8,32).
wap("A waiter had 7 tables he was waiting on , with 7 women and 2 men at each table . How many customers total did the waiter have ?",7,7,2,63).
wap("Will was working as a sacker at a grocery store where he made 8 dollars an hour . On Monday he worked 8 hours and on Tuesday he worked 2 hours . How much money did Will make in those two days ?",8,8,2,80).
wap("Rachel bought 7 new chairs and 3 new tables for her house . If she spent 4 minutes on each piece furniture putting it together , how many minutes did it take her to finish ?",7,3,4,40).
wap("While playing a trivia game , Adam answered 8 questions correct in the first half and 2 questions correct in the second half . If each question was worth 8 points , what was his final score ?",8,2,8,80).
wap("Billy was putting his spare change into piles . He had 2 piles of quarters and 3 piles of dimes . If each pile had 4 coins in it , how many coins did he have total ?",2,3,4,20).
wap("Frank and his friend were buying trick decks from the magic shop for 7 dollars each . How much did they spend if Frank bought 3 decks and his friend bought 2 decks ?",7,3,2,35).
wap("A waiter had 5 tables he was waiting on , with 5 women and 3 men at each table . How many customers total did the waiter have ?",5,5,3,40).
wap("While playing a trivia game , George answered 6 questions correct in the first half and 4 questions correct in the second half . If each question was worth 3 points , what was his final score ?",6,4,3,30).
wap("At the town carnival Oliver rode the ferris wheel 7 times and the bumper cars 3 times . If each ride cost 3 tickets , how many tickets did he use ?",7,3,3,30).
wap("Kaleb was collecting cans for recycling . On Saturday he filled 4 bags up and on Sunday he filled 6 more bags . If each bag had 4 cans in it , how many cans did he pick up total ?",4,6,4,40).
wap("A waiter had 9 tables he was waiting on , with 2 women and 6 men at each table . How many customers total did the waiter have ?",9,2,6,72).
wap("There were 4 friends playing a video game online when 5 more players joined the game . If each player had 3 lives , how many lives did they have total ?",4,5,3,27).
wap("The schools debate team had 26 boys and 46 girls on it . If they were split into groups of 9 how many groups could they make ?",26,46,9,8).
wap("Robin uploaded 35 pictures from her phone and 5 from her camera to facebook . If she sorted the pics into 5 different albums with the same amount of pics in each album , how many pictures were in each of the albums ?",35,5,5,8).
wap("For homework Nancy had 17 math problems and 15 spelling problems . If she can finish 8 problems in an hour how long will it take her to finish all the problems ?",17,15,8,4).
wap("A pet shelter had 9 puppies when another 12 were brought in . If 3 puppies a day are adopted , how long would it take for all of them to be adopted ?",9,12,3,7).
wap("While playing at the arcade , Frank won 33 tickets playing ` whack a mole ' and 9 tickets playing ` skee ball ' . If he was trying to buy candy that cost 6 tickets a piece , how many could he buy ?",33,9,6,7).
wap("For Halloween Megan received 11 pieces of candy from neighbors and 5 pieces from her older sister . If she only ate 8 pieces a day , how long would the candy last her ?",11,5,8,2).
wap("Maria was making baggies of cookies with 5 cookies in each bag . If she had 33 chocolate chip cookies and 2 oatmeal cookies , how many baggies could she make ?",5,33,2,7).
wap("A pet shelter had 5 puppies when another 35 were brought in . If 8 puppies a day are adopted , how long would it take for all of them to be adopted ?",5,35,8,5).
wap("Dave was helping the cafeteria workers pick up lunch trays , but he could only carry 9 trays at a time . If he had to pick up 17 trays from one table and 55 trays from another , how many trips will he make ?",9,17,55,8).
wap("Maria was making baggies of cookies with 8 cookies in each bag . If she had 5 chocolate chip cookies and 19 oatmeal cookies , how many baggies could she make ?",8,5,19,3).
wap("Paul made 68 dollars mowing lawns and 13 dollars weed eating . If he only spent 9 dollar a week , how long would the money last him ?",68,13,9,9).
wap("While playing at the arcade , Edward won 3 tickets playing ` whack a mole ' and 5 tickets playing ` skee ball ' . If he was trying to buy candy that cost 4 tickets a piece , how many could he buy ?",3,5,4,2).
wap("Luke was organizing his baseball cards in a binder with 3 on each page . If he had 3 new cards and 9 old cards to put in the binder , how many pages would he use ?",3,3,9,4).
wap("Olivia uploaded 5 pictures from her phone and 35 from her camera to facebook . If she sorted the pics into 8 different albums with the same amount of pics in each album , how many pictures were in each of the albums ?",5,35,8,5).
wap("A pet shelter had 17 puppies when another 55 were brought in . If 8 puppies a day are adopted , how long would it take for all of them to be adopted ?",17,55,8,9).
wap("A toy store had 6 giant stuffed bears in stock when they got another shipment with 18 bears in it . The put the bears onto shelves with 6 on each shelf . How many shelves did they use ?",6,18,6,4).
wap("A pet shelter had 17 puppies when another 55 were brought in . If 8 puppies a day are adopted , how long would it take for all of them to be adopted ?",17,55,8,9).
wap("There school 's baseball team had 12 new players and 44 returning players . If the coach put them into groups with 8 players in each group , how many groups would there be ?",12,44,8,7).
wap("Bianca and her friends were recycling paper for their class . For every 3 pounds they recycled they earned one point . If Bianca recycled 24 pounds and her friends recycled 3 pounds , how many points did they earn ?",3,24,3,9).
wap("Oliver was organizing his baseball cards in a binder with 3 on each page . If he had 2 new cards and 10 old cards to put in the binder , how many pages would he use ?",3,2,10,4).
wap("Paul made 3 dollars mowing lawns and 3 dollars weed eating . If he only spent 3 dollar a week , how long would the money last him ?",3,3,3,2).
wap("The schools debate team had 5 boys and 40 girls on it . If they were split into groups of 9 how many groups could they make ?",5,40,9,5).
wap("At a company picnic 23 managers and 7 employees decided to start a game of volleyball . If they split into 6 teams how many people would be on each team ?",23,7,6,5).
wap("For a birthday party Cody bought 4 regular sodas and 44 diet sodas . If his fridge would only hold 6 on each shelf , how many shelves would he fill up ?",4,44,6,8).
wap("Ned was helping the cafeteria workers pick up lunch trays , but he could only carry 5 trays at a time . If he had to pick up 5 trays from one table and 5 trays from another , how many trips will he make ?",5,5,5,2).
wap("For homework Amy had 18 math problems and 6 spelling problems . If she can finish 4 problems in an hour how long will it take her to finish all the problems ?",18,6,4,6).
wap("Maria was making baggies of cookies with 3 cookies in each bag . If she had 2 chocolate chip cookies and 16 oatmeal cookies , how many baggies could she make ?",3,2,16,6).
wap("Nancy 's old washing machine could only wash 9 pieces of clothing at a time . If she had to wash 19 shirts and 8 sweaters how many loads would she have to do ?",9,19,8,3).
wap("Jerry made 14 dollars mowing lawns and 31 dollars weed eating . If he only spent 5 dollar a week , how long would the money last him ?",14,31,5,9).
wap("Zoe and her friends were recycling paper for their class . For every 8 pounds they recycled they earned one point . If Zoe recycled 25 pounds and her friends recycled 23 pounds , how many points did they earn ?",8,25,23,6).
wap("Victor was helping the cafeteria workers pick up lunch trays , but he could only carry 7 trays at a time . If he had to pick up 23 trays from one table and 5 trays from another , how many trips will he make ?",7,23,5,4).
wap("Frank made 5 dollars mowing lawns and 58 dollars weed eating . If he only spent 7 dollar a week , how long would the money last him ?",5,58,7,9).
wap("Ned bought 14 boxes of chocolate candy and gave 7 to his little brother . If each box has 6 pieces inside it , how many pieces did Ned still have ?",14,7,6,42).
wap("There were 11 friends playing a video game online when 5 players quit . If each player left had 5 lives , how many lives did they have total ?",11,5,5,30).
wap("Each chocolate bar in a box cost $ 3 . If a box had 7 bars total and Olivia sold all but 4 bars , how much money would she have made ?",3,7,4,9).
wap("At a restaurant each adult meal costs $ 3 and kids eat free . If a group of 12 people came in and 7 were kids , how much would it cost for the group to eat ?",3,12,7,15).
wap("Ned had 15 video games but 6 of them were n't working . If he wanted to sell the working games for $ 7 each , how much money could he earn ?",15,6,7,63).
wap("In a video game , each enemy defeated gives you 5 points . If a level has 8 enemies total and you destroy all but 6 of them , how many points would you earn ?",5,8,6,10).
wap("A new building needed 9 windows . The builder had already installed 6 of them . If it takes 6 hours to install each window , how long will it take him to install the rest ?",9,6,6,18).
wap("Dave bought 12 boxes of chocolate candy and gave 5 to his little brother . If each box has 3 pieces inside it , how many pieces did Dave still have ?",12,5,3,21).
wap("John had 5 action figures , but needed 7 total for a complete collection . If each one costs $ 5 , how much money would he need to finish his collection ?",5,7,5,10).
wap("A worksheet had 4 problems on it . If a teacher had 9 worksheets to grade and had already graded 5 of them , how many more problems does she have to grade ?",4,9,5,16).
wap("A magician was selling magic card decks for 9 dollars each . If he started with 12 decks and by the end of the day he had 7 left , how much money did he earn ?",9,12,7,45).
wap("A chef needs to cook 9 potatoes . He has already cooked 7 . If each potato takes 3 minutes to cook , how long will it take him to cook the rest ?",9,7,3,6).
wap("A painter needed to paint 12 rooms in a building . Each room takes 3 hours to paint . If he already painted 4 rooms , how much longer will he take to paint the rest ?",12,3,4,24).
wap("At a restaurant each adult meal costs $ 7 and kids eat free . If a group of 13 people came in and 9 were kids , how much would it cost for the group to eat ?",7,13,9,28).
wap("A trivia team had 7 members total , but during a game 2 members did n't show up . If each member that did show up scored 4 points , how many points were scored total ?",7,2,4,20).
wap("A chef needs to cook 15 potatoes . He has already cooked 6 . If each potato takes 8 minutes to cook , how long will it take him to cook the rest ?",15,6,8,72).
wap("John earned 8 dollars for each lawn he mowed . If he had 15 lawns to mow , but forgot to mow 7 of them , how much money did he actually earn ?",8,15,7,64).
wap("Dave had 10 video games but 2 of them were n't working . If he wanted to sell the working games for $ 4 each , how much money could he earn ?",10,2,4,32).
wap("At a restaurant each adult meal costs $ 5 and kids eat free . If a group of 15 people came in and 8 were kids , how much would it cost for the group to eat ?",5,15,8,35).
wap("A chef needs to cook 15 potatoes . He has already cooked 8 . If each potato takes 9 minutes to cook , how long will it take him to cook the rest ?",15,8,9,63).
wap("A trivia team had 14 members total , but during a game 7 members did n't show up . If each member that did show up scored 5 points , how many points were scored total ?",14,7,5,35).
wap("A trivia team had 11 members total , but during a game 6 members did n't show up . If each member that did show up scored 9 points , how many points were scored total ?",11,6,9,45).
wap("Each chocolate bar in a box cost $ 2 . If a box had 13 bars total and Rachel sold all but 4 bars , how much money would she have made ?",2,13,4,18).
wap("At lunch a waiter had 7 customers and 4 of them did n't leave a tip . If he got $ 9 each from the ones who did tip , how much money did he earn ?",7,4,9,27).
wap("Tom bought 14 boxes of chocolate candy and gave 8 to his little brother . If each box has 3 pieces inside it , how many pieces did Tom still have ?",14,8,3,18).
wap("At a restaurant each adult meal costs $ 2 and kids eat free . If a group of 15 people came in and 9 were kids , how much would it cost for the group to eat ?",2,15,9,12).
wap("John had 6 action figures , but needed 11 total for a complete collection . If each one costs $ 6 , how much money would he need to finish his collection ?",6,11,6,30).
wap("At a restaurant each adult meal costs $ 8 and kids eat free . If a group of 11 people came in and 2 were kids , how much would it cost for the group to eat ?",8,11,2,72).
wap("At the fair Kaleb bought 6 tickets . After riding the ferris wheel he had 3 tickets left . If each ticket cost 9 dollars , how much money did Kaleb spend riding the ferris wheel ?",6,3,9,27).
wap("A magician was selling magic card decks for 7 dollars each . If he started with 16 decks and by the end of the day he had 8 left , how much money did he earn ?",7,16,8,56).
wap("At the fair Adam bought 10 tickets . After riding the ferris wheel he had 3 tickets left . If each ticket cost 9 dollars , how much money did Adam spend riding the ferris wheel ?",10,3,9,63).
wap("A worksheet had 7 problems on it . If a teacher had 17 worksheets to grade and had already graded 8 of them , how many more problems does she have to grade ?",7,17,8,63).
wap("Edward earned 9 dollars for each lawn he mowed . If he had 6 lawns to mow , but forgot to mow 2 of them , how much money did he actually earn ?",9,6,2,36).
wap("Chloe baked 7 brownies , but needed 16 total for her party . If she used 6 cups of flour on each one , how much cups of flour does she still need ?",7,16,6,54).
wap("Sam invited 9 friends to a birthday party , but 6 could n't come . If he wanted to buy enough cupcakes so each person could have exactly 2 , how many should he buy ?",9,6,2,6).
wap("Megan earned 8 points for each bag of cans she recycled . If she had 14 bags , but did n't recycle 5 of them , how many points would she have earned ?",8,14,5,72).
wap("Wendy uploaded 79 pictures to Facebook . She put 44 pics into one album and put the rest into 5 different albums . How many pictures were in each album ?",79,44,5,7).
wap("Emily was planting vegetables in her garden . She started with 41 seeds and planted 29 of them in the big garden and in each of her small gardens put 4 seeds each . How many small gardens did Emily have ?",41,29,4,3).
wap("For Halloween Sarah received 108 pieces of candy . She ate 36 pieces then placed the rest into piles with 9 in each pile . How many piles could she make ?",108,36,9,8).
wap("Nancy had 80 files on her computer . She deleted 31 of them and put the rest into folders with 7 files in each one . How many folders did Nancy end up with ?",80,31,7,7).
wap("There are 36 students trying out for the school 's trivia teams . If 9 of them did n't get picked for the team and the rest were put into 3 groups , how many students would be in each group ?",36,9,3,9).
wap("A pet store had 64 puppies . In one day they sold 28 of them and put the rest into cages with 4 in each cage . How many cages did they use ?",64,28,4,9).
wap("Frank made 19 dollars mowing lawns over the summer . If he spent 11 dollars buying new mower blades , how many 2 dollar games could he buy with the money he had left ?",19,11,2,4).
wap("A pet store had 78 puppies . In one day they sold 30 of them and put the rest into cages with 8 in each cage . How many cages did they use ?",78,30,8,6).
wap("A waiter had 44 customers in his section . If 12 of them left and the rest of his tables had 8 people at each table , how many tables did he have ?",44,12,8,4).
wap("Kaleb was selling his old games . He started out with 76 but sold 46 of them . He packed the rest up putting 5 games into each box . How many boxes did he have to use ?",76,46,5,6).
wap("Nancy was planting vegetables in her garden . She started with 52 seeds and planted 28 of them in the big garden and in each of her small gardens put 4 seeds each . How many small gardens did Nancy have ?",52,28,4,6).
wap("A waiter had 21 customers in his section . If 12 of them left and the rest of his tables had 3 people at each table , how many tables did he have ?",21,12,3,3).
wap("There are 17 students trying out for the school 's trivia teams . If 5 of them did n't get picked for the team and the rest were put into 3 groups , how many students would be in each group ?",17,5,3,4).
wap("Megan baked 71 cupcakes for her school 's bake sale . If her brother , Todd , ate 43 of them how many packages could she make if she put 7 cupcake in each package ?",71,43,7,4).
wap("Roger had 120 pieces of clothing to wash . He put 48 of them in one load , but decided to split the rest into 9 equal loads . How many pieces of clothing could go in each of the small loads ?",120,48,9,8).
wap("Isabel baked 39 cupcakes for her school 's bake sale . If her brother , Todd , ate 21 of them how many packages could she make if she put 3 cupcake in each package ?",39,21,3,6).
wap("There are 58 students trying out for the school 's trivia teams . If 10 of them did n't get picked for the team and the rest were put into 8 groups , how many students would be in each group ?",58,10,8,6).
wap("A pet store had 13 puppies . In one day they sold 7 of them and put the rest into cages with 2 in each cage . How many cages did they use ?",13,7,2,3).
wap("Kaleb had 12 dollars . If he spent 8 bucks on a new game , how many 2 dollar toys could he buy with the money he had left ?",12,8,2,2).
wap("Mike made 69 dollars mowing lawns over the summer . If he spent 24 dollars buying new mower blades , how many 5 dollar games could he buy with the money he had left ?",69,24,5,9).
wap("Will made 104 dollars mowing lawns over the summer . If he spent 41 dollars buying new mower blades , how many 9 dollar games could he buy with the money he had left ?",104,41,9,7).
wap("Luke was selling his old games . He started out with 57 but sold 39 of them . He packed the rest up putting 2 games into each box . How many boxes did he have to use ?",57,39,2,9).
wap("The cafeteria had 75 apples . For lunch they handed out 19 to students and decided to use the rest to make pies . If each pie takes 8 apples , how many pies could they make ?",75,19,8,7).
wap("Sarah baked 38 cupcakes for her school 's bake sale . If her brother , Todd , ate 14 of them how many packages could she make if she put 8 cupcake in each package ?",38,14,8,3).
wap("Nancy uploaded 41 pictures to Facebook . She put 37 pics into one album and put the rest into 2 different albums . How many pictures were in each album ?",41,37,2,2).